/*
 * wxcdde.cc
 * wxWindows DDE command processing
 *
 */

#include "wx.h"
#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#include "clips.h"
#endif

extern "C" int clipsEchoOn;
extern ExtensionErrorClass ExtensionError;

/************************************************************************
 * DDE
 *
 ************************************************************************/

Bool wxClipsConnectionOnAdvise(Bool *handled, WXID id, char *topic, char *item, char *data, int size, int format)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnAdvise");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnAdvise \"%s\" \"%s\" \"%s\"))", func, id, topic, item, data);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Connection OnAdvise callback function should return an integer (0 or 1).\n";
      return FALSE;
    }
    else 
      return (Bool)clipsReturnLong;
  }
  return FALSE;
}

Bool wxClipsConnectionOnExecute(Bool *handled, WXID id, char *topic, char *data, int size, int format)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnExecute");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnExecute \"%s\" \"%s\" \"%s\"))", func, id, topic, "no item", data);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Connection OnExecute callback function should return an integer (0 or 1).\n";
      return FALSE;
    }
    else 
      return (Bool)clipsReturnLong;
  }
  return FALSE;
}

Bool wxClipsConnectionOnPoke(Bool *handled, WXID id, char *topic, char *item, char *data, int size, int format)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnPoke");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnPoke \"%s\" \"%s\" \"%s\"))", func, id, topic, item, data);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Connection OnPoke callback function should return an integer (0 or 1).\n";
      return FALSE;
    }
    else 
      return (Bool)clipsReturnLong;
  }
  return FALSE;
}

char *wxClipsConnectionOnRequest(Bool *handled, WXID id, char *topic, char *item, int *size, int format)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnRequest");
  if (func)
  {
    *handled = TRUE;

#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnRequest \"%s\" \"%s\" \"%s\"))", func, id, topic, item, "no data");
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsSTRING)
    {
      ExtensionError << "Connection OnRequest callback function should return a string.\n";
      return NULL;
    }
    else 
      return clipsReturnString;
  }
  return NULL;
}

Bool wxClipsConnectionOnStartAdvise(Bool *handled, WXID id, char *topic, char *item)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnStartAdvise");
  if (func)
  {
    *handled = TRUE;

#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnStartAdvise \"%s\" \"%s\" \"%s\"))", func, id, topic, item, "no data");
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Connection OnStartAdvise callback function should return an integer (0 or 1).\n";
      return FALSE;
    }
    else 
      return (Bool)clipsReturnLong;
  }
  return FALSE;
}

Bool wxClipsConnectionOnStopAdvise(Bool *handled, WXID id, char *topic, char *item)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnStopAdvise");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld OnStopAdvise \"%s\" \"%s\" \"%s\"))", func, id, topic, item, "no data");
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Connection OnStopAdvise callback function should return an integer (0 or 1).\n";
      return FALSE;
    }
    else 
      return (Bool)clipsReturnLong;
  }
  return FALSE;
}

Bool wxClipsServerOnAcceptConnection(Bool *handled, WXID id,
    char *topic, WXID connectionId)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnAcceptConnection");
  if (func)
  {
    *handled = TRUE;
        
#ifdef USE_CLIPS
    char buf[400];
    sprintf(buf, "(return-result (%s %ld \"%s\" %ld))", func, id, topic, connectionId);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "Server OnAcceptConnection callback function should return an integer.\n";
      return FALSE;
    }
    else return (Bool)clipsReturnLong;
#else
    return FALSE;
#endif
  }
  return FALSE;
}

/*
 * DDE functions
 *
 */

#ifdef USE_CLIPS
long clipsClientCreate(void)
{
//  int no_args = RtnArgCount();
  if (ArgCountCheck("clipsClientCreate", EXACTLY, 0) == -1)
    return 0;

  return wxcClientCreate();
}

/*
 * Returns an integer representing the connection, or zero if failed.
 *
 */
 
long clipsClientMakeConnection(void)
{
//  int no_args = RtnArgCount();
  if (ArgCountCheck("clipsClientMakeConnection", EXACTLY, 4) == -1)
    return 0;

  long id = RtnLong(1);
  char *host = RtnLexeme(2);
  char *service = RtnLexeme(3);
  char *topic = RtnLexeme(4);

  if (!host || !service || !topic)
    return 0;

  return wxcClientMakeConnection(id, host, service, topic);
}

long clipsServerCreate(void)
{
//  int no_args = RtnArgCount();
  if (ArgCountCheck("clipsServerCreate", EXACTLY, 1) == -1)
    return 0;

  char *service = RtnLexeme(1);
  if (!service)
    return 0;
  return wxcServerCreate(service);
}

/*
 * Connection
 */
 
long clipsConnectionAdvise(void)
{
  if (ArgCountCheck("clipsConnectionAdvise", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  char *item = RtnLexeme(2);
  char *data = RtnLexeme(3);

  if (!item || !data)
    return 0;
  return (long)wxcConnectionAdvise(id, item, data);
}

long clipsConnectionExecute(void)
{
  if (ArgCountCheck("clipsConnectionExecute", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *data = RtnLexeme(2);

  if (!data)
    return 0;
  return (long)wxcConnectionExecute(id, data);
}

long clipsConnectionDisconnect(void)
{
  if (ArgCountCheck("clipsConnectionDisconnect", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcConnectionDisconnect(id);
}

long clipsConnectionPoke(void)
{
  if (ArgCountCheck("clipsConnectionPoke", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  char *item = RtnLexeme(2);
  char *data = RtnLexeme(3);

  if (!item || !data)
    return 0;
  return (long)wxcConnectionPoke(id, item, data);
}

char *clipsConnectionRequest(void)
{
  if (ArgCountCheck("clipsConnectionRequest", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *item = RtnLexeme(2);

  if (!item)
    return (char *)AddSymbol("");

  char *data = wxcConnectionRequest(id, item);
  if (data)
    return (char *)AddSymbol(data);
  else
    return (char *)AddSymbol("");
}

long clipsConnectionStartAdvise(void)
{
  if (ArgCountCheck("clipsConnectionStartAdvise", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *item = RtnLexeme(2);

  if (!item)
    return 0;
  return (long)wxcConnectionStartAdvise(id, item);
}

long clipsConnectionStopAdvise(void)
{
  if (ArgCountCheck("clipsConnectionStopAdvise", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *item = RtnLexeme(2);

  if (!item)
    return 0;

  return wxcConnectionStopAdvise(id, item);
}
#endif

/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS
void wxDDEUserFunctions()
{  
  DefineFunction("client-create", 'l', PTIF clipsClientCreate, "clipsClientCreate");
  DefineFunction("client-make-connection", 'l', PTIF clipsClientMakeConnection, "clipsClientMakeConnection");

  DefineFunction("server-create", 'l', PTIF clipsServerCreate, "clipsServerCreate");

  DefineFunction("connection-advise", 'l', PTIF clipsConnectionAdvise, "clipsConnectionAdvise");
  DefineFunction("connection-execute", 'l', PTIF clipsConnectionExecute, "clipsConnectionExecute");
  DefineFunction("connection-disconnect", 'l', PTIF clipsConnectionDisconnect, "clipsConnectionDisconnect");
  DefineFunction("connection-poke", 'l', PTIF clipsConnectionPoke, "clipsConnectionPoke");
  DefineFunction("connection-request", 's', PTIF clipsConnectionRequest, "clipsConnectionRequest");
  DefineFunction("connection-start-advise", 'l', PTIF clipsConnectionStartAdvise, "clipsConnectionStartAdvise");
  DefineFunction("connection-stop-advise", 'l', PTIF clipsConnectionStopAdvise, "clipsConnectionStopAdvise");

}
#endif

