/*
 * gdi.cc
 * wxWindows command processing: GDI objects
 *
 */

#include "wx.h"
#include "wx_hash.h"
#include "wx_types.h"

#ifdef wx_msw
// #if LOADABLE_BITMAPS
#include "dib.h"
// #endif
// #if LOADABLE_ICONS
#include "wxcurico.h"
// #endif
#endif
#if defined(wx_x)// && USE_IMAGE_LOADING_IN_X
#include "../../image/src/wx_image.h"
#endif

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#include "clips.h"
#endif

extern ExtensionErrorClass ExtensionError;

#ifdef USE_CLIPS
long clipsPenCreate()
{
  if (ArgCountCheck("clipsPenCreate", EXACTLY, 3) == -1)
    return 0;

  char *colour = RtnLexeme(1);
  long width = RtnLong(2);
  char *style = RtnLexeme(3);

  return wxcPenCreate(colour, (int)width, style);
}

long clipsPenDelete()
{
  if (ArgCountCheck("clipsPenDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcPenDelete(id);
}

long clipsBrushCreate()
{
  if (ArgCountCheck("clipsBrushCreate", EXACTLY, 2) == -1)
    return 0;

  char *colour = RtnLexeme(1);
  char *style = RtnLexeme(2);
  return wxcBrushCreate(colour, style);
}

long clipsBrushDelete()
{
  if (ArgCountCheck("clipsBrushDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcBrushDelete(id);
}

long clipsFontCreate()
{
  if (ArgCountCheck("clipsFontCreate", EXACTLY, 5) == -1)
    return 0;

  int pointSize = (int)RtnLong(1);
  char *family = RtnLexeme(2);
  char *style = RtnLexeme(3);
  char *weight  = RtnLexeme(4);
  Bool underlined = (Bool)RtnLong(5);
  return wxcFontCreate(pointSize, family, style, weight, underlined);
}

long clipsFontDelete()
{
  if (ArgCountCheck("clipsFontDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcFontDelete(id);
}

long clipsBitmapCreate()
{
  if (ArgCountCheck("clipsBitmapCreate", AT_LEAST, 2) == -1)
    return 0;

  int depth = -1;
  if (RtnArgCount() > 2)
    depth = (int)RtnLong(3);
  return wxcBitmapCreate((int)RtnLong(1), (int)RtnLong(2), depth);
}

long clipsBitmapLoadFromFile()
{
  if (ArgCountCheck("clipsBitmapLoadFromFile", AT_LEAST, 1) == -1)
    return 0;
#ifdef wx_msw
  char *bitmapType = "wxBITMAP_TYPE_BMP";
#else
  char *bitmapType = "wxBITMAP_TYPE_XBM";
#endif
  if (RtnArgCount() > 1)
    bitmapType = RtnLexeme(2);
  return wxcBitmapLoadFromFile(RtnLexeme(1), bitmapType);
}

long clipsBitmapDelete()
{
  if (ArgCountCheck("clipsBitmapDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcBitmapDelete(id);
}

long clipsBitmapGetWidth()
{
  if (ArgCountCheck("clipsBitmapGetWidth", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcBitmapGetWidth(id);
}

long clipsBitmapGetHeight()
{
  if (ArgCountCheck("clipsBitmapGetHeight", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcBitmapGetHeight(id);
}

long clipsBitmapGetColourMap()
{
  if (ArgCountCheck("clipsBitmapGetColourMap", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcBitmapGetColourMap(id);
}

long clipsIconCreate()
{
  if (ArgCountCheck("clipsIconCreate", EXACTLY, 1) == -1)
    return 0;

  char *file = RtnLexeme(1);
  return wxcIconCreate(file);
}

long clipsIconLoadFromFile()
{
  if (ArgCountCheck("clipsIconLoadFromFile", AT_LEAST, 1) == -1)
    return 0;
#ifdef wx_msw
  char *bitmapType = "wxBITMAP_TYPE_ICO";
#else
  char *bitmapType = "wxBITMAP_TYPE_XBM";
#endif
  if (RtnArgCount() > 1)
    bitmapType = RtnLexeme(2);

  return wxcIconLoadFromFile(RtnLexeme(1), bitmapType);
}

long clipsIconDelete()
{
  if (ArgCountCheck("clipsIconDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcIconDelete(id);
}

long clipsIconGetWidth()
{
  if (ArgCountCheck("clipsIconGetWidth", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcIconGetWidth(id);
}

long clipsIconGetHeight()
{
  if (ArgCountCheck("clipsIconGetHeight", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcIconGetHeight(id);
}

long clipsCursorCreate()
{
  if (ArgCountCheck("clipsCursorCreate", EXACTLY, 1) == -1)
    return 0;

  char *cursorType = RtnLexeme(1);
  if (!cursorType)
    return 0;
  return wxcCursorCreate(cursorType);
}

long clipsCursorLoadFromFile()
{
  if (ArgCountCheck("clipsCursorLoadFromFile", AT_LEAST, 1) == -1)
    return 0;
#ifdef wx_msw
  char *bitmapType = "wxBITMAP_TYPE_CUR";
#else
  char *bitmapType = "wxBITMAP_TYPE_XBM";
#endif
  int x = 0;
  int y = 0;
  if (RtnArgCount() > 1)
    bitmapType = RtnLexeme(2);
  if (RtnArgCount() > 2)
    x = (int)RtnLong(3);
  if (RtnArgCount() > 3)
    y = (int)RtnLong(4);

  return wxcCursorLoadFromFile(RtnLexeme(1), bitmapType, x, y);
}

long clipsCursorDelete()
{
  if (ArgCountCheck("clipsCursorDelete", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcCursorDelete(id);
}
#endif

#ifdef USE_CLIPS
void wxGDIUserFunctions()
{
  DefineFunction("bitmap-create", 'l', PTIF clipsBitmapCreate, "clipsBitmapCreate");
  DefineFunction("bitmap-load-from-file", 'l', PTIF clipsBitmapLoadFromFile, "clipsBitmapLoadFromFile");
  DefineFunction("bitmap-delete", 'l', PTIF clipsBitmapDelete, "clipsBitmapDelete");
  DefineFunction("bitmap-get-width", 'l', PTIF clipsBitmapGetWidth, "clipsBitmapGetWidth");
  DefineFunction("bitmap-get-height", 'l', PTIF clipsBitmapGetHeight, "clipsBitmapGetHeight");
  DefineFunction("bitmap-get-colourmap", 'l', PTIF clipsBitmapGetColourMap, "clipsBitmapGetColourMap");

  DefineFunction("icon-create", 'l', PTIF clipsIconCreate, "clipsIconCreate");
  DefineFunction("icon-load-from-file", 'l', PTIF clipsIconLoadFromFile, "clipsIconLoadFromFile");
  DefineFunction("icon-delete", 'l', PTIF clipsIconDelete, "clipsIconDelete");
  DefineFunction("icon-get-width", 'l', PTIF clipsIconGetWidth, "clipsIconGetWidth");
  DefineFunction("icon-get-height", 'l', PTIF clipsIconGetHeight, "clipsIconGetHeight");

  DefineFunction("cursor-create", 'l', PTIF clipsCursorCreate, "clipsCursorCreate");
  DefineFunction("cursor-load-from-file", 'l', PTIF clipsCursorLoadFromFile, "clipsCursorLoadFromFile");
  DefineFunction("cursor-delete", 'l', PTIF clipsCursorDelete, "clipsCursorDelete");

  DefineFunction("brush-create", 'l', PTIF clipsBrushCreate, "clipsBrushCreate");
  DefineFunction("brush-delete", 'l', PTIF clipsBrushDelete, "clipsBrushDelete");

  DefineFunction("font-create", 'l', PTIF clipsFontCreate, "clipsFontCreate");
  DefineFunction("font-delete", 'l', PTIF clipsFontDelete, "clipsFontDelete");

  DefineFunction("pen-create", 'l', PTIF clipsPenCreate, "clipsPenCreate");
  DefineFunction("pen-delete", 'l', PTIF clipsPenDelete, "clipsPenDelete");
}
#endif
