/*
 * wxctbar.cc
 * wxWindows command processing: toolbar
 *
 */

#include "wx.h"
#include "wx_hash.h"
#include "wx_types.h"

#include "wx_tbar.h"

#ifdef wx_msw
#include "wx_bbar.h"
#endif

#include <iostream.h>
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#include "clips.h"
#endif

#include "wxctbarp.h"

extern "C" int clipsEchoOn;
extern ExtensionErrorClass ExtensionError;

Bool wxClipsToolBarClassOnLeftClick(Bool *handled, WXID id, int toolIndex, Bool toggleDown)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnLeftClick");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    clipsReturnType = clipsUNKNOWN;
    sprintf(buf, "(return-result (%s %ld %ld %ld))", func, id, (long)toolIndex, (long)toggleDown);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
    clipsReturnLong = (* (long (*)(long, int, Bool)) func) (id, (long)toolIndex, toggleDown);
    clipsReturnType = clipsLONG;
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "OnLeftClick callback for toolbar should return an integer (1 or 0)!\n";
      return FALSE;
    }
    else return (Bool)clipsReturnLong;
  }
  return TRUE;
}

void wxClipsToolBarClassOnRightClick(Bool *handled, WXID id, int toolIndex, float x, float y)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnRightClick");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(return-result (%s %ld %ld %.2f %.2f))", func, id, (long)toolIndex, x, y);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
    (* (void (*)(long, int, double, double)) func) (id, toolIndex, (double)x, (double)y);
#endif
  }
}

void wxClipsToolBarClassOnMouseEnter(Bool *handled, WXID id, int toolIndex)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnMouseEnter");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(return-result (%s %ld %ld))", func, id, (long)toolIndex);
    clipsEchoOn = FALSE;
    RouteCommand(buf);
    clipsEchoOn = TRUE;
#else
    (* (void (*)(long, Bool)) func) ((long)this, toolIndex);
#endif
  }
}

#ifdef USE_CLIPS
long clipsToolbarCreate()
{
  if (ArgCountCheck("clipsToolbarCreate", AT_LEAST, 1) == -1)
    return 0;

  int no_args = RtnArgCount();

  long parent_id = RtnLong(1);

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *orientation = "wxVERTICAL";
  int rowsOrColumns = 2;
  Bool createButtons = 1;

  if (no_args > 1)
    x = (int) RtnLong(2);

  if (no_args > 2)
    y = (int) RtnLong(3);

  if (no_args > 3)
    width = (int) RtnLong(4);

  if (no_args > 4)
    height = (int) RtnLong(5);

  if (no_args > 5)
    style = RtnLexeme(6);

  if (no_args > 6)
    orientation = RtnLexeme(7);

  if (no_args > 7)
    rowsOrColumns = (int)RtnLong(8);

  if (no_args > 8)
    createButtons = (Bool)RtnLong(9);

  return wxcToolbarCreate(parent_id, x, y, width, height, style, orientation, rowsOrColumns, createButtons);
}

long clipsToolbarAddTool()
{
  if (ArgCountCheck("clipsToolbarAddTool", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  int index = (int)RtnLong(2);
  long bitmapId1 = RtnLong(3);
  long bitmapId2 = 0;
  Bool toggle = FALSE;
  double x = -1.0;
  double y = -1.0;
  long clientData = 0;
  
  int no_args = RtnArgCount();

  if (no_args > 3)
    bitmapId2 = (int) RtnLong(4);

  if (no_args > 4)
    toggle = (Bool) RtnLong(5);

  if (no_args > 5)
    x = RtnDouble(6);

  if (no_args > 6)
    y = RtnDouble(7);

  if (no_args > 7)
    clientData = RtnLong(8);

  return (long)wxcToolbarAddTool(id, index, bitmapId1, bitmapId2, toggle, x, y, clientData);
}

long clipsToolbarClearTools()
{
  if (ArgCountCheck("clipsToolbarClearTools", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarClearTools(RtnLong(1));
}

long clipsToolbarEnableTool()
{
  if (ArgCountCheck("clipsToolbarEnableTool", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarEnableTool(RtnLong(1), (int)RtnLong(2), (Bool)RtnLong(3));
}

long clipsToolbarToggleTool()
{
  if (ArgCountCheck("clipsToolbarToggleTool", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarToggleTool(RtnLong(1), (int)RtnLong(2), (Bool)RtnLong(3));
}

long clipsToolbarLayout()
{
  if (ArgCountCheck("clipsToolbarLayout", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarLayout(RtnLong(1));
}

double clipsToolbarGetMaxWidth()
{
  if (ArgCountCheck("clipsToolbarGetMaxWidth", EXACTLY, 1) == -1)
    return 0;

  return wxcToolbarGetMaxWidth(RtnLong(1));
}

double clipsToolbarGetMaxHeight()
{
  if (ArgCountCheck("clipsToolbarGetMaxHeight", EXACTLY, 1) == -1)
    return 0;

  return wxcToolbarGetMaxHeight(RtnLong(1));
}

long clipsToolbarOnPaint()
{
  if (ArgCountCheck("clipsToolbarOnPaint", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarOnPaint(RtnLong(1));
}

long clipsToolbarGetToolClientData()
{
  if (ArgCountCheck("clipsToolbarGetToolClientData", EXACTLY, 2) == -1)
    return 0;

  return wxcToolbarGetToolClientData(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarGetToolEnabled()
{
  if (ArgCountCheck("clipsToolbarGetToolEnabled", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcToolbarGetToolEnabled(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarGetToolState()
{
  if (ArgCountCheck("clipsToolbarGetToolState", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcToolbarGetToolState(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarSetMargins()
{
  if (ArgCountCheck("clipsToolbarSetMargins", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetMargins(RtnLong(1), RtnDouble(2), RtnDouble(3));
}

long clipsToolbarSetDefaultSize()
{
  if (ArgCountCheck("clipsToolbarSetDefaultSize", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetDefaultSize(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}
#endif

#ifdef USE_CLIPS
void wxToolbarUserFunctions()
{
  DefineFunction("toolbar-add-tool", 'l', PTIF clipsToolbarAddTool, "clipsToolbarAddTool");
  DefineFunction("toolbar-clear-tools", 'l', PTIF clipsToolbarClearTools, "clipsToolbarClearTools");
  DefineFunction("toolbar-create", 'l', PTIF clipsToolbarCreate, "clipsToolbarCreate");
  DefineFunction("toolbar-enable-tool", 'l', PTIF clipsToolbarEnableTool, "clipsToolbarEnableTool");
  DefineFunction("toolbar-toggle-tool", 'l', PTIF clipsToolbarToggleTool, "clipsToolbarToggleTool");
  DefineFunction("toolbar-get-tool-state", 'l', PTIF clipsToolbarGetToolState, "clipsToolbarGetToolState");
  DefineFunction("toolbar-get-tool-enabled", 'l', PTIF clipsToolbarGetToolEnabled, "clipsToolbarGetToolEnabled");
  DefineFunction("toolbar-get-max-width", 'd', PTIF clipsToolbarGetMaxWidth, "clipsToolbarGetMaxWidth");
  DefineFunction("toolbar-get-max-height", 'd', PTIF clipsToolbarGetMaxHeight, "clipsToolbarGetMaxHeight");
  DefineFunction("toolbar-get-tool-client-data", 'l', PTIF clipsToolbarGetToolClientData,
     "clipsToolbarGetToolClientData");
  DefineFunction("toolbar-layout", 'l', PTIF clipsToolbarLayout, "clipsToolbarLayout");
  DefineFunction("toolbar-on-paint", 'l', PTIF clipsToolbarOnPaint, "clipsToolbarOnPaint");
  DefineFunction("toolbar-set-margins", 'l', PTIF clipsToolbarSetMargins, "clipsToolbarSetMargins");
  DefineFunction("toolbar-set-default-size", 'l', PTIF clipsToolbarSetDefaultSize, "clipsToolbarSetDefaultSize");
}
#endif
